#ifndef __GLOBAL_TRIGGER_INHIBIT_QUEUES_H
#define __GLOBAL_TRIGGER_INHIBIT_QUEUES_H

#include <sys/types.h>
#include <string>
#include <deque>

#include <boost/thread/mutex.hpp>

#include <NovaGlobalTrigger/GT.h>
#include <NovaGlobalTrigger/GT_MessageQueue.h>

#include <DAQMessages/ccpp_GTMessages.h>

// GT_NAMESPACE_BEGIN

/************************************************************************
 * Impimentation of a single Inhibit Queue for the Nova Global Trigger  *
 *                                                                      *
 * This implimentation is derived off of the message FIFO and adds      *
 * functions for checking for overlaps between a test message and the   *
 * messages already in the queue                                        *
 ************************************************************************/
class inhibitQueue: public messageQueue {

 public:

  inhibitQueue();
  // ~inhibitQueue(){};

  bool checkInhibit(triggermessages::TriggerMessage theTrig);   // Returns true if the passed message overlaps with an inhibit region

  enum inhibitCheckType {
    Inhibit_No_Overlap
    , Inhibit_Any_Overlap
    , Inhibit_Contained_Overlap
    , Inhibit_Complete_Overlap
    , Inhibit_ForwardStraddle_Overlap
    , Inhibit_RearStraddle_Overlap
  };
  
 private:


}; // endclass messageQueue

// GT_NAMESPACE_END

#endif /* __GLOBAL_TRIGGER_INHIBIT_QUEUES_H */
