#ifndef __GLOBAL_TRIGGER_MESSAGE_QUEUES_H
#define __GLOBAL_TRIGGER_MESSAGE_QUEUES_H

#include <sys/types.h>
#include <string>
#include <deque>

#include <boost/thread/mutex.hpp>

#include <NovaGlobalTrigger/GT.h>

#include <DAQMessages/ccpp_GTMessages.h>

// #include <DAQMessages/ccpp_DDTMessages.h>
// #include <NovaGlobalTrigger/GTGlobals.h>
// #include <NovaGlobalTrigger/GTLongTrigs.h>
// #include <NovaGlobalTrigger/GTInfo.h>
// #include <messagefacility/MessageLogger/MessageLogger.h>
// #include <DAQDataFormats/TriggerDefines.h>
// #include <NovaTimingUtilities/TimingUtilities.h>

// GT_NAMESPACE_BEGIN

/************************************************************************
 * Impimentation of a single Message Queue for the Nova Global Trigger  *
 *                                                                      *
 * This implimentation is done as a FIFO using std::deque               *
 * Note: This implimentation uses boost::mutexes to make the deque      *
 * thread safe                                                          *
 *                                                                      *
 * There is an optional length parameter that changes the queue to be   *
 * a fixed depth.                                                       *
 ************************************************************************/
class messageQueue {

 public:

  messageQueue();
  // ~messageQueue(){};

  bool queueMessage(triggermessages::TriggerMessage& newMessage);  //! Push the trigger message into the queue

  //triggermessages::TriggerMessage & popMessage();     //! Pop the top trigger message off the queue
  triggermessages::TriggerMessage* popMessage();     //! Pop the top trigger message off the queue

  size_t size();                                      //! Current size of the queue
  bool empty();                                       //! Returns true if the queue empty
  void flush();                                       //! Flush the queue out
  void reset();                                       //! Full reset of the queue
  int getDroppedCount();                              //! Returns the count of the number of triggers that were dropped

  int setMaxQueueDepth(int value);                    //! Configure the maximum depth of the queue

 protected:

  deque<triggermessages::TriggerMessage> theQueue;    // The Queue implimented as a deque
  
 private:

  int max_queue_depth;                                // Maximum Size of the queue (set negative for unlimited)
  int drop_count;                                     // Number of dropped messages

  boost::mutex theQueue_mutex;                        // Mutex for locking the queue

}; // endclass messageQueue

// GT_NAMESPACE_END

#endif /* __GLOBAL_TRIGGER_MESSAGE_QUEUES_H */
