#ifndef __GT_STATS_H_
#define __GT_STATS_H_

#include <sys/types.h>
#include <boost/thread/mutex.hpp>
#include <vector>

#include <NovaDAQMonitorClient/NdmcClient.h>

#include <NovaGlobalTrigger/GT.h>

#include <NovaGlobalTrigger/GTInfo.h>

GT_NAMESPACE_BEGIN

// the class definition cannot be extern
// so it must be put in there manually when the instance is declared
// in the non-instantiating files
class GT_Stats {

public:

  // set this up for Singleton-ness
  static GT_Stats& getInstance();

  // What's going on with all possible triggers.  Exposing this array directly
  // rather than methodizing the thing simply to preserve my sanity.
  // This declares the array.  Will give it the size in the .cpp file
  // which will create the objects.
  std::vector<GTInfo> Triggers;

  // Ganglia master pointer.  Needs to be accessed from all over
  nova::novadaqmonitorclient::NdmcClient _daqMonClient; // daq monitor client

  // methods
  uint64_t getTrigNo();                   // get the current trigger #
  uint64_t incTrigNo();                   // increments the trigger #

  uint64_t reportTrigNo(uint64_t[], uint32_t); // dump trigger stats
  void    resetTrigNo();                 // resets trigger # and all counts

  void    initSpillMetrics();            // create spill ganglia metrics
  void    initSNEWSMetrics();            // create SNEWS ganglia metrics
  void    logSpillDelay(double);         // log spill trigger delay to ganglia
  void    logSNEWSDelay(double);         // log SNEWS trigger delay to ganglia
  void    startHeartbeat();              // start ganglia partition heartbeat
  void    stopHeartbeat();               // stop ganglia partition heartbeat

private:

  explicit GT_Stats();   // Constructor
  //      ~GT_Stats();   // Destructor not needed?

  // disallow any copying
  GT_Stats(GT_Stats const&);  // not implemented
  void operator= (GT_Stats const&);  // not implemented

  // data
  uint64_t TriggerNumber;
  boost::mutex TriggerNumber_mutex;

  // Ganglia metrics
  int32_t TriggerRate;
  int32_t spillTriggerDelayMAX;
  int32_t spillTriggerDelayAVG;
  int32_t spillTriggerDelayMIN;
  int32_t spillTriggerDelaySTD;
  int32_t snewsTriggerDelayMAX;
  int32_t snewsTriggerDelayAVG;
  int32_t snewsTriggerDelayMIN;
  int32_t snewsTriggerDelaySTD;
  // Heartbeat for the monitor client
  int32_t _monIdPartitionHeartbeat;

  // methods
};

GT_NAMESPACE_END

#endif // __GT_STATS_H_
