#ifndef __NGTMCSTATEMACHINE_H_
#define __NGTMCSTATEMACHINE_H_

#include <boost/thread/mutex.hpp>

#include <NovaGlobalTrigger/GT.h>

GT_NAMESPACE_BEGIN

/**
 * NGTMCStateMachine
 *
 * This class implements a simple state machine to manage transitions 
 * between NGTMC States.
 *
 * Stolen from Sue's DcmStateMachine
 */

class NGTMCStateMachine {

public:

  // enums and typedefs
  typedef enum NGTMCState {
    NO_PARTITION,
    PARTITION_ESTABLISHED,
    RUN_CONFIGURATION_LOADED,
    RUN_CONFIGURED
  } NGTMCState_t;

  // static methods
  static std::string NGTMCStateAsString(NGTMCState_t state);

  // constructors/destructor
  NGTMCStateMachine ();
  virtual ~NGTMCStateMachine ();

  // state changing methods
  bool changeState(NGTMCState_t state, std::string configURI = "", 
		   int32_t partition = -1);

  // accessor methods
  NGTMCState_t getCurrentState(int32_t partition);

private:

  // Data
  NGTMCState_t _currentState[NUMPARTS]; // current state

  // Private pure virtual methods
  virtual bool _establishPartition(int32_t partition) = 0;
  virtual bool _loadRunConfiguration(std::string configURI, 
				     int32_t partition) = 0;
  virtual bool _configureRun(int32_t partition) = 0;
  virtual bool _deconfigureRun(int32_t partition) = 0;
  virtual void _unloadRunConfiguration(int32_t partition) = 0;
  virtual bool _detachFromPartition(int32_t partition) = 0;

  // Private methods used by changeState
  bool _changeStateFromNoPartition(NGTMCState_t state, 
				   std::string configURI, int32_t partition);
  bool _changeStateFromPartitionEstablished(NGTMCState_t state, 
					    std::string configURI,
                                            int32_t partition = -1);
  bool _changeStateFromRunConfigurationLoaded(NGTMCState_t state, 
					      std::string configURI, 
					      int32_t partition);
  bool _changeStateFromRunConfigured(NGTMCState_t state, 
				     std::string configURI, int32_t partition);
};

GT_NAMESPACE_END

#endif
