#ifndef _NGT_RC_H
#define _NGT_RC_H

#include <NovaDAQUtilities/Runnable.h>
#include <NovaRunControlClient/RunControlReceiver.h>
#include <DAQMessages/ccpp_RunControlMessages.h>
#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <messagefacility/MessageLogger/MessageLogger.h>

#include <NovaGlobalTrigger/NGTStateMachine.h>

#include <NovaGlobalTrigger/GT.h>

GT_NAMESPACE_BEGIN

class NGTRC : public novadaq::Runnable, public NGTStateMachine {

public:

  /**
   * Creates an instance with the specified application name.
   */
  NGTRC(std::string applicationName);

  /**
   * Destructor to clean up any allocated memory.
   */
  ~NGTRC();

  /**
   * Contains the operations that are executed in a separate thread.
   */
  void run();

  /**
   * Handle run control messages
   */
  boost::shared_ptr<runcontrolmessages::BeginRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::BeginRunRequest>
	          message);

  boost::shared_ptr<runcontrolmessages::StopRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::StopRunRequest>
	          message);

  boost::shared_ptr<runcontrolmessages::PauseRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::PauseRunRequest>
		  message);

  boost::shared_ptr<runcontrolmessages::ResumeRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::ResumeRunRequest>
		  message);

  boost::shared_ptr<runcontrolmessages::StatusResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::StatusRequest>
		  message);

  boost::shared_ptr<runcontrolmessages::RolloverRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::RolloverRunRequest>
		  message);

private:
  // variables
  std::string _applicationName;
  boost::shared_ptr<rcclient::RunControlReceiver> _rcReceiver;
  boost::shared_ptr<mf::Presence> _mfPresence;
  mutable boost::mutex _callbackMutex;

  // methods which do stuff upon state changes
  bool _beginRun();
  bool _pauseRun();
  void _resumeRun();
  void _stopRun();
  uint64_t _rolloverRun(uint64_t[], uint32_t);
};

GT_NAMESPACE_END

#endif
