#ifndef _RUN_CONTROL_RECEIVER_H
#define _RUN_CONTROL_RECEIVER_H

#include <NovaRunControlClient/RmsMessageClient.h>

namespace novadaq {

namespace rcclient {

/**
 * The run control receiver handles the receipt of messages from
 * and the sending of replies to run control.  It does this by
 * allowing user code to specify callbacks that get invoked when
 * specified message types arrive.
 *
 * @author Kurt Biery
 * @version $Revision: 1.9 $ $Date: 2011/09/23 13:41:34 $
 */

class RunControlReceiver : public RmsMessageClient {

 public:

    RunControlReceiver(std::string applicationName,
                       std::vector<std::string> bcastTargetList,
                       bool provideDefaultHandlers = true);

    ~RunControlReceiver();

 protected:

    bool _thisInstanceShouldInstallDefaultHandlers();

    void _addDefaultListeners(DDSConnectionPtr ddsConnection,
                              RMSNS::base::RmsDestination destination,
                              int partitionNumber,
                              ReceiverWrapperListPtr receiverListPtr);

 private:

    static boost::mutex _instanceMutex;
    static std::vector<int64_t> _instanceList;
};

};  // end of namespace rcclient

};  // end of namespace novadaq

#endif
