#ifndef DDSMAILBOX__H
#define DDSMAILBOX__H

#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>

#include <rms/provider/DDSConnection.h>
#include <rms/base/RmsDestination.h>
#include <rms/RmsReceiver.h>
#include <rms/RmsSender.h>

using namespace gov::fnal::cd::rms;

/**
* @brief Virtual dds mailbox class - a wrapper for DDSConnection
*
* @tparam MessageType - a message class to be sent or received
*/
template <typename MessageType> class DDSMailbox{
  public:
     
      /**
      * @brief establish connection with given ConnectionName, partition number and MailboxName
      *
      * @param ConnectionName
      * @param RmsPartition
      * @param MailboxName):
      */
      DDSMailbox(std::string ConnectionName,int RmsPartition, std::string MailboxName):
           _rms_connection(boost::make_shared<provider::DDSConnection>(ConnectionName,RmsPartition)),
           _rms_dest(MailboxName,base::RmsDestination::TRIGGER_CHANNEL) {}
      virtual ~DDSMailbox() {};
  protected:
    boost::shared_ptr<provider::DDSConnection> _rms_connection; /*< handle for DDSConnection*/
    base::RmsDestination _rms_dest; /*< Connection destination*/
};

/**
* @brief DDSInbox: used for reading messages from given mailbox with >> operator
*
* @tparam MessageType - a message class to be received
*/
template <typename MessageType> class DDSInbox: public DDSMailbox<MessageType>{
  public:
      DDSInbox(std::string ConnectionName,int RmsPartition, std::string MailboxName):
              DDSMailbox<MessageType>(ConnectionName,RmsPartition,MailboxName),
              _rms_receiver(this->_rms_connection,this->_rms_dest) {};
      ~DDSInbox() {}
      
      /**
      * @brief receive message
      *
      */
      DDSInbox<MessageType> & operator >> (MessageType& message){
            _rms_receiver.receiveMessage(message);
            return *this;
      }
   private:
       RmsReceiver < provider::DDSConnection, MessageType > _rms_receiver;
};

/**
* @brief DDSOutbox - sends messages to given Mailbox destination with << operator
*
* @tparam MessageType - a message class to be received
*/
template <typename MessageType> class DDSOutbox: public DDSMailbox<MessageType>{
   public:
      DDSOutbox(std::string ConnectionName,int RmsPartition, std::string MailboxName):
              DDSMailbox<MessageType>(ConnectionName,RmsPartition,MailboxName),
              _rms_sender(this->_rms_connection,this->_rms_dest) {};
      ~DDSOutbox() {}

      DDSOutbox <MessageType>& operator << (MessageType& message){
            _rms_sender.sendMessage(message);
            return *this;
      }
   private:
       RmsSender < provider::DDSConnection, MessageType > _rms_sender;
};

#endif

