#ifndef FIRFILTER__H
#define FIRFILTER__H
#include <boost/shared_array.hpp>
#include "NovaSuperNova/SNBase.h"
#include "NovaSuperNova/SNDataBuffer.h"
namespace supernova{
/**
* @file FIRFilter.h
* @brief A class for filtering buffer data
* Main operation: convolves a buffer with given kernel (@see convolve())
* for usage example see unittest/testFilter.cc
* @author Andrey Sheshukov
* @version 1
* @date 2015-04-17
*/
    class FIRFilter{
      public:
        //default ctor
        FIRFilter():_length(0),_kernel(0){}
        //ctor with empty kernel of size N
        FIRFilter(size_t N);
        //ctor with given kernel
        FIRFilter(size_t N, double* K);

        //access to n-th kernel point
        double& operator[](size_t n){return _kernel[n];}
        const double& operator[](size_t n) const {return _kernel[n];}
        //get kernel size
        size_t size(){return _length;}
        //calculate norm - sum of all kernel points
        double norm();
        //rescale to make norm=1
        void normalize(); 
        //calculate the result of convolution of buffer(starting from given point) with kernel
        double convolve (const BufferIterator& point) const;
      private:
        size_t _length;  /*< Kernel size */
        boost::shared_array<double> _kernel; /*< Kernel */
    };

}
#endif
