#ifndef SNDATACHUNK__H
#define SNDATACHUNK__H

#include <iostream>
#include <algorithm>
#include <boost/shared_array.hpp>
#include "NovaSuperNova/SNVirtualDataBlock.h"
#include "NovaSuperNova/SNExceptions.h"

namespace supernova{

/**
* @brief DataBlock is piece of data buffer, continous in time. 
* It stores data for time interval [t0;t1) in fixed number of bins.
*/
class DataBlock: public VirtualDataBlock{
  public:

 	DataBlock(size_t Length,const TimeRange& range);
	~DataBlock();

  // bin size (duration)
	TimeType dt() const;

	//access data at given position
  DataType& dataAtPos(size_t n){return _data[n];}
  //calculate time for given position
	TimeType timeAtPos(size_t n);
  //access data at given time
	DataType& dataAtTime(const TimeType &t){return _data[findPosition(t)];}

  //find bin, corresponding to given time
	size_t findPosition (const TimeType &t)const;

  //get number of times, a block had been filled
	size_t nentries() const {return _entries;}
  //check if all data bins were filled 
  bool   full() const {return _entries>=_length;}

	virtual bool empty() const{return _entries==0;}
	virtual size_t size() const {return _length;}
	virtual void fill(const TimeType& t, const DataType& d);
  private:
	const size_t _length;
	size_t _entries;
	boost::shared_array <DataType> _data;
};

// stream output operator
std::ostream& operator<<(std::ostream& stream, DataBlock& block);
}

#endif
