#ifndef SNEXCEPTIONS__H
#define SNEXCEPTIONS__H

#include <stdexcept>
#include <sstream>

namespace supernova{
    namespace errors{
        /**
        * @brief base class for supernova errors
        */
        class basic_error: public std::logic_error{
           public:
               basic_error(const std::string& message):std::logic_error(message){};
        };
        /**
        * @brief out of range errors: when trying to access data outside given range
        */
        class out_of_range: public basic_error{
           public:
          out_of_range(TimeType t,TimeType t1, TimeType t2):basic_error(makeMessage(t,t1,t2)){};
          static std::string makeMessage(TimeType t,TimeType t0, TimeType t1){
                std::stringstream msg;
                msg<<"Point "<<t<<" out of range ["<<t0<<","<<t1<<"]";
                return msg.str();
             }
        };

        /**
        * @brief fill collision exception: trying to fill data into nonempty bin
        */
        class fill_collision: public basic_error{
           public:	
              fill_collision(size_t pos,DataType d_to_fill, DataType d_filled):basic_error(makeMessage(pos,d_to_fill,d_filled)){};
              static std::string makeMessage(size_t pos,DataType d_to_fill, DataType d_filled){
                std::stringstream msg;
                msg<<"Filling data="<<d_to_fill<<" collides with data["<<pos<<"]="<<d_filled;
                return msg.str();
             }
        };

    }

}
#endif
