#include <boost/thread/thread.hpp>

#include <NovaGlobalTrigger/GTCalTrig.h>
#include <NovaGlobalTrigger/GTStreamerTrig.h>
#include <NovaGlobalTrigger/GTSpillTrig.h>
#include <NovaGlobalTrigger/GTSimTrig.h>
#include <NovaGlobalTrigger/GTDataTrig.h>
#include <NovaGlobalTrigger/GTSNEWSTrig.h>
#include <NovaGlobalTrigger/GTSuperNovaListener.h>
#include <NovaGlobalTrigger/GTSuperNovaTrig.h>
#include <NovaGlobalTrigger/GTManualTrig.h>
#include <NovaGlobalTrigger/GTInit.h>
#include <NovaGlobalTrigger/GT_Stats.h>
#include <NovaGlobalTrigger/GTSendTrigAgent.h>
#define DEFINE_GLOBALS
#include <NovaGlobalTrigger/GTGlobals.h>

// for rcclient                                                                                          
#include <NovaGlobalTrigger/NGTRC.h>

#include <messagefacility/MessageLogger/MessageLogger.h>

#include <rms/base/RmsDestination.h>

// for cmd-line parsing
#include <unistd.h> 
#include <getopt.h>

//#include <dlfnc.h>

using namespace std;
using namespace novadaq;
using namespace gt;

//------------------------------------------------------------

int main(int argc, char* argv[]) {

  // Attach to message logging service here. 
  // this uses the multi-threaded version, logs to console, and to 
  // the message facility on the appropriate partition
  std::string msgFacCfgFile = "GlobalTriggerMsgFac.fcl";
  mf::StartMessageFacility( mf::MessageFacilityService::MultiThread, 
			    mf::MessageFacilityService::
                            ConfigurationFile(msgFacCfgFile.c_str(),
                                              mf::MessageFacilityService::
                                              logCS()));
  mf::SetApplicationName("NovaGlobalTrigger");
  mf::SetModuleName("main");
  //  mf::SetContext("pre-event");  // what does this do?

  /* Initialize, read in config, etc */
  GTInit(argc,argv);

  // Attach to DAQ monitor here.  Send RMS message to RC claiming we're alive?

  // create the Run Control Client application and start it
  novadaq::gt::NGTRC NGTRC("GlobalTrigger");
  boost::thread      gtRCThread(boost::bind(&novadaq::Runnable::run, &NGTRC));

  // Create a singleton instance to keep track of trigger numbers.
  gt::GT_Stats::getInstance();

  // Now go do stuff.  For each trigger, spin off a thread.
  // If they turn out to not be enabled, the thread will return.
  
  // Calibration pulser Trigger Thread 
  TriggerGenerator* theCalTrigHandler = static_cast<TriggerGenerator*>( new GTCalTrig());
  theCalTrigHandler->configure();
  boost::thread GTCalTrigThrd( &TriggerGenerator::work, theCalTrigHandler );

  // Streamer pulser Trigger Thread 
  TriggerGenerator* theStreamerTrigHandler = static_cast<TriggerGenerator*>( new GTStreamerTrig());
  theStreamerTrigHandler->configure();
  boost::thread GTStreamerTrigThrd( &TriggerGenerator::work, theStreamerTrigHandler );

  // Beam Spill Trigger Thread 
  TriggerGenerator* theSpillTrigHandler = static_cast<TriggerGenerator*>( new GTSpillTrig());
  theSpillTrigHandler->configure();
  boost::thread GTSpillTrigThrd( &TriggerGenerator::work, theSpillTrigHandler );

  // spin off a manual trigger thread
  TriggerGenerator* theManualTrigHandler = static_cast<TriggerGenerator*>( new GTManualTrig());
  theManualTrigHandler->configure();
  boost::thread GTManualTrigThrd( &TriggerGenerator::work, theManualTrigHandler );

  // Data Driven Trigger Handler
  TriggerGenerator* theDDTHandler = static_cast<TriggerGenerator*>( new GTDataTrig());
  theDDTHandler->configure();
  boost::thread GTDataTrigThrd( &TriggerGenerator::work, theDDTHandler );

  // SNEWS Trigger Handler
  TriggerGenerator* theSNEWSHandler = static_cast<TriggerGenerator*>( new GTSNEWSTrig());
  theSNEWSHandler->configure();
  boost::thread GTSNEWSTrigThrd( &TriggerGenerator::work, theSNEWSHandler );

  // SuperNova triggers Trigger Handler
  GTSuperNovaTrig*     SNTrigHandler=new GTSuperNovaTrig();
  GTSuperNovaListener* SNListHandler=new GTSuperNovaListener();
  SNTrigHandler->configure();
  SNListHandler->configure();
 //connect output buffer of listener with input buffer of trigger 
  SNListHandler->setOutputBuffer(SNTrigHandler->Buffer());
  boost::thread GTNSNTrigThrd( &TriggerGenerator::work, 
           static_cast<TriggerGenerator*>(SNTrigHandler));
  boost::thread GTNSNListThrd( &TriggerGenerator::work, 
           static_cast<TriggerGenerator*>(SNListHandler));


  // spin off a simulation trigger thread
  boost::thread GTSimTrigThrd(&GTSimTrig);

  /****************************************************************************
   * Start the Agent that Handles the Trigger Queues                          *
   ****************************************************************************/
  boost::thread GTSendTrigThread(&GTSendTrigAgent);

  /****************************************************************************
   * Thread Sync Fence                                                        *
   ****************************************************************************/

  // Wait for cal thread to finish
  GTCalTrigThrd.join();
  // Wait for Streamer thread to finish
  GTStreamerTrigThrd.join();
  // Wait for spill thread to finish
  GTSpillTrigThrd.join();
  // Wait for manual thread to finish
  GTManualTrigThrd.join();
  // Wait for manual thread to finish
  GTSimTrigThrd.join();
  // Wait for datadriven trigger to finish
  GTDataTrigThrd.join();
  // Wait for SNEWS trigger to finish
  GTSNEWSTrigThrd.join();
  // Wait for SuperNova triggers to finish
  GTNSNTrigThrd.join();
  GTNSNListThrd.join();
  // Wait for the Sender Thread to join (which I don't think it actually can do)
  // GTSendTrigThread.join();

  gtRCThread.join();

  // tell ganglia we're done.  Should be done in the GT_Stats destructor?
  GT_Stats::getInstance()._daqMonClient.setPartition(gov::fnal::cd::rms::base::RmsDestination::NULL_PARTITION);

  return 0;
}
