#include <NovaRunControlClient/GlobalTriggerReceiver.h>

namespace novadaq {

namespace rcclient {

boost::mutex GlobalTriggerReceiver::_instanceMutex;
std::vector<int64_t> GlobalTriggerReceiver::_instanceList;

/**
 * Creates a receiver instance with the specified application name and
 * list of broadcast targets to listen to.
 */
GlobalTriggerReceiver::
GlobalTriggerReceiver(std::string applicationName,
                      std::vector<std::string> bcastTargetList) :
    RmsMessageClient(applicationName, bcastTargetList,
                     RMSNS::base::RmsDestination::TRIGGER_CHANNEL,
                     false)
{
    // add this instance to this instanceList
    {
        boost::mutex::scoped_lock sl(_instanceMutex);
        _instanceList.push_back((int64_t) this);
    }
}

/**
 * Destructor.
 */
GlobalTriggerReceiver::~GlobalTriggerReceiver()
{
    // remove this instance from this instanceList
    {
        boost::mutex::scoped_lock sl(_instanceMutex);
        for (uint32_t idx = 0; idx < _instanceList.size(); ++idx) {
            if (_instanceList[idx] == (int64_t) this) {
                _instanceList.erase(_instanceList.begin()+idx);
                break;
            }
        }
    }
}

/**
 * Tests if the current instance should install default handlers.
 */
bool GlobalTriggerReceiver::_thisInstanceShouldInstallDefaultHandlers()
{
    return false;
}

/**
 * Adds the full set of default listeners to this receiver instance
 * for the specified destination.
 */
void GlobalTriggerReceiver::
_addDefaultListeners(DDSConnectionPtr ddsConnection,
                     RMSNS::base::RmsDestination destination,
                     int partitionNumber,
                     ReceiverWrapperListPtr receiverListPtr) {
    // no default handlers
}

};  // end of namespace rcclient

};  // end of namespace novadaq
