#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

//! Daemon Program Initialization
/*!
 * Daemon_init will detach the process and properly
 * promote it to daemon status where it will have no
 * controlling TTY and will not be the leader of a 
 * process group etc...
 *
 * This routine should be called after program startup
 */
int daemon_init(void){

  pid_t pid;			//! Program PID
  
  // Attempt to fork the process
  if((pid = fork()) < 0){	// Check for fork success
    return(-1);			// return on failure
  }
  else{
    if(pid !=0){		// If we are the parent after the fork
      exit(0);			// then exit
    }
  } // endif fork

  //! Parent Exits
  //! Child continues execution flow
  
  setsid();			// Become the session leader

  // Now Fork a second time
  if((pid = fork()) < 0){	// Check for fork success
    return(-1);			// return on failure
  }
  else{
    if(pid !=0){		// If we are the parent after the fork
      exit(0);			// then exit
    }
  } // endif fork
 
  // We are the second child and can not possibly have a controlling terminal

  return(0);			// Return with success

} // end daemon_init

