#include "NovaSuperNova/FIRFilter.h"
#include <stdio.h>
using namespace supernova;

/**
* @brief ctor with empty kernel of size N
*
* @param N - kernel lenght
*/
FIRFilter::FIRFilter(size_t N):_length(N),_kernel(new double[_length]){
    std::fill(_kernel.get(),_kernel.get()+_length,0);
};

/**
* @brief ctor with given kernel
*
* @param N - kernel lenght
* @param K - pointer to array with kernel points
*/
FIRFilter::FIRFilter(size_t N, double* K):_length(N),_kernel(new double[_length]){
    //copy given array to kernel
    std::copy(K,K+N,_kernel.get());
};

/**
* @brief calculate kernel norm - sum of all kernel points
*
* @return sum of all kernel points 
*/
double FIRFilter::norm(){
   double result=0;
   for(size_t n=0; n<_length; ++n){
      result+=_kernel[n];
   }
   return result;
}

/**
* @brief rescale to make norm=1. Skip if norm=0
*/
void FIRFilter::normalize(){
   double Norm=norm();
   if(Norm==0)return;
   for(size_t n=0; n<_length; ++n){_kernel[n]/=Norm;}
}

/**
* @brief calculate the result of convolution of buffer(starting from given point) with kernel
*
* @param start - iterator to DataBuffer point, which is the beginning of convolution (shift)
*
* @return result of convolution
*/
double FIRFilter::convolve(const BufferIterator& start) const{
   double result=0;
   BufferIterator buffer_point=start;
   //printf("convolving:\n");
   for(size_t n=0; n<_length; ++n){
      double product=_kernel[n]*buffer_point.value(); 
      //printf("[%ld] %4.3g * %ld = %4.3g \n",n,_kernel[n],buffer_point.value(),product);
      result+=product;
      ++buffer_point;
   }
   return result;
}
