
#include "NovaSuperNova/SNDataBlock.h"

using namespace supernova;
/**
* @brief Construct empty DataBlock with given parameters
*
* Data array is filled with zeros.
*
* @param Length - block length (number of bins)
* @param TimeRange - block time limits
*/
DataBlock::DataBlock (size_t Length=0,const TimeRange &range=TimeRange(0,0)):
                   VirtualDataBlock(range),_length(Length),_entries(0),
                  _data(new DataType[_length]){
 	std::fill(_data.get(),_data.get()+_length,0);
};

DataBlock::~DataBlock() {
}


/**
* @brief Fill data into bin,which corresponds to given timestamp
* @throws errors::fill_collision if this bin is not empty (not zero)
* @throws errors::out_of_range if timestamp is outside time limits 
*
* @param t - timestamp
* @param d - data to be filled
*/
void DataBlock::fill(const TimeType& t, const DataType& d){
          size_t position=findPosition(t);
          if(_data[position]!=0) throw errors::fill_collision(position,d,_data[position]);
          _data[position]=d;
          _entries++;
}

/**
* @brief Calculate bin size (duration)
*
* @return bin size
*/
TimeType DataBlock::dt() const {
	return duration()/_length;
}


/**
* @brief find bin, corresponding to given timestamp
* @throws errors::out_of_range if timestamp is outside time limits
* @param t - timestamp
*
* @return bin position
*/
size_t DataBlock::findPosition (const TimeType &t)const{
          if(!this->contains(t))throw errors::out_of_range(t,startTime(),endTime());
          return (t-startTime())/dt();
    }


/**
* @brief Calculate time of the given bin position
*
* @param n - bin position 
*
* @return Time of bin start
*/
TimeType DataBlock::timeAtPos(size_t n){
	return startTime()+n*dt();
    }


std::ostream& supernova::operator <<(std::ostream &stream, DataBlock& block){
   stream<<"DataBlock: size"<<block.size()<<"; duration="<<block.duration()<<std::endl;
   stream<<"time interval=["<<block.startTime()<<" - "<<block.endTime()
         <<"], filled="<<block.nentries()<<"/"<<block.size()<<std::endl;
   for(size_t n=0;n<block.size();++n)
      stream<<block[n]<<std::endl;
   return stream;
}
